@@ -27,6 +27,9 @@ static bool hasCmdLineFSanitizeFuzzer = false;
 
 /* Embed libhf/.a inside this binary */
 __asm__("\n"
+#if !defined(_HF_ARCH_DARWIN) && !defined(__APPLE__)
+        "   .section .rodata\n"
+#endif /* _HF_ARCH_DARWIN */
         "   .global lhfuzz_start\n"
         "   .global lhfuzz_end\n"
         "lhfuzz_start:\n"
@@ -147,6 +150,7 @@ static int execCC(int argc, char** argv) {
     }
     if (useUBSAN()) {
         argv[argc++] = "-fsanitize=undefined";
+        argv[argc++] = "-fno-sanitize-recover=undefined";
     }
     argv[argc] = NULL;
 
@@ -177,6 +181,12 @@ static int execCC(int argc, char** argv) {
         if (isCXX) {
             /* Try the default one, then the newest ones (hopefully) in order */
             hf_execvp("clang++", argv);
+            hf_execvp("clang++-16.0", argv);
+            hf_execvp("clang++-16", argv);
+            hf_execvp("clang++-15.0", argv);
+            hf_execvp("clang++-15", argv);
+            hf_execvp("clang++-14.0", argv);
+            hf_execvp("clang++-14", argv);
             hf_execvp("clang++-13.0", argv);
             hf_execvp("clang++-13", argv);
             hf_execvp("clang++-12.0", argv);
@@ -201,6 +211,12 @@ static int execCC(int argc, char** argv) {
         } else {
             /* Try the default one, then the newest ones (hopefully) in order */
             hf_execvp("clang", argv);
+            hf_execvp("clang-16.0", argv);
+            hf_execvp("clang-16", argv);
+            hf_execvp("clang-15.0", argv);
+            hf_execvp("clang-15", argv);
+            hf_execvp("clang-14.0", argv);
+            hf_execvp("clang-14", argv);
             hf_execvp("clang-13.0", argv);
             hf_execvp("clang-13", argv);
             hf_execvp("clang-12.0", argv);
@@ -321,7 +337,8 @@ static char* getLibHFCommonPath() {
     if (path[0]) {
         return path;
     }
-    if (!getLibPath("libhfuzz", "HFUZZ_LHFCOMMON_PATH", &lhfcommon_start, &lhfcommon_end, path)) {
+    if (!getLibPath(
+            "libhfcommon", "HFUZZ_LHFCOMMON_PATH", &lhfcommon_start, &lhfcommon_end, path)) {
         LOG_F("Couldn't create the temporary libhcommon.a");
     }
     return path;
@@ -438,6 +455,7 @@ static int ldMode(int argc, char** argv) {
     args[j++] = "-Wl,--wrap=bcmp";
     args[j++] = "-Wl,--wrap=memmem";
     args[j++] = "-Wl,--wrap=strcpy";
+    args[j++] = "-Wl,--wrap=strlcpy";
     /* Apache httpd */
     args[j++] = "-Wl,--wrap=ap_cstr_casecmp";
     args[j++] = "-Wl,--wrap=ap_cstr_casecmpn";
