@@ -196,6 +196,39 @@ static inline char* HF_strcpy(char* dest, const char* src, uintptr_t addr) {
     return __builtin_memcpy(dest, src, len + 1);
 }
 
+static inline char* HF_strcat(char* dest, const char* src, uintptr_t addr) {
+    size_t len = __builtin_strlen(dest);
+    return HF_strcpy(dest + len, src, addr);
+}
+
+static inline size_t HF_strlcpy(char* dest, const char* src, size_t sz, uintptr_t addr) {
+    size_t slen = __builtin_strlen(src);
+    size_t len  = sz < slen ? sz : slen;
+
+    if (sz == 0) {
+        return 0;
+    }
+    /* Make space for NUL at the end of the string.
+     * sz != 0 here
+     */
+    if (len == sz) {
+        len--;
+    }
+
+    if (len > 0) {
+        instrumentUpdateCmpMap(addr, util_Log2(len));
+        (void)__builtin_memcpy(dest, src, len);
+    }
+
+    dest[len] = '\0';
+    return len;
+}
+
+static inline size_t HF_strlcat(char* dest, const char* src, size_t sz, uintptr_t addr) {
+    size_t len = __builtin_strlen(dest);
+    return HF_strlcpy(dest + len, src, sz, addr);
+}
+
 /* Define a weak function x, as well as __wrap_x pointing to x */
 #define XVAL(x) x
 #define HF_WEAK_WRAP(ret, func, ...)                                                               \
@@ -290,6 +323,27 @@ void __sanitizer_weak_hook_strcpy(
     uintptr_t pc, char* dest, const char* src, char* result HF_ATTR_UNUSED) {
     HF_strcpy(dest, src, pc);
 }
+HF_WEAK_WRAP(char*, strcat, char* dest, const char* src) {
+    return HF_strcat(dest, src, (uintptr_t)__builtin_return_address(0));
+}
+void __sanitizer_weak_hook_strcat(
+    uintptr_t pc, char* dest, const char* src, char* result HF_ATTR_UNUSED) {
+    HF_strcat(dest, src, pc);
+}
+HF_WEAK_WRAP(size_t, strlcpy, char* dest, const char* src, size_t len) {
+    return HF_strlcpy(dest, src, len, (uintptr_t)__builtin_return_address(0));
+}
+void __sanitizer_weak_hook_strlcpy(
+    uintptr_t pc, char* dest, const char* src, size_t sz, size_t result HF_ATTR_UNUSED) {
+    HF_strlcpy(dest, src, sz, pc);
+}
+HF_WEAK_WRAP(size_t, strlcat, char* dest, const char* src, size_t len) {
+    return HF_strlcat(dest, src, len, (uintptr_t)__builtin_return_address(0));
+}
+void __sanitizer_weak_hook_strlcat(
+    uintptr_t pc, char* dest, const char* src, size_t sz, size_t result HF_ATTR_UNUSED) {
+    HF_strlcat(dest, src, sz, pc);
+}
 
 /*
  * Apache's httpd wrappers
