@@ -73,10 +73,12 @@
 #define MAX_INSTR_SZ 8
 #elif defined(__mips__) || defined(__mips64__)
 #define MAX_INSTR_SZ 8
+#elif defined(__riscv)
+#define MAX_INSTR_SZ 4
 #endif
 
 #if defined(__i386__) || defined(__x86_64__)
-struct user_regs_struct_32 {
+struct user_regs_32 {
     uint32_t ebx;
     uint32_t ecx;
     uint32_t edx;
@@ -96,7 +98,7 @@ struct user_regs_struct_32 {
     uint16_t ss, __ss;
 };
 
-struct user_regs_struct_64 {
+struct user_regs_64 {
     uint64_t r15;
     uint64_t r14;
     uint64_t r13;
@@ -125,40 +127,50 @@ struct user_regs_struct_64 {
     uint64_t fs;
     uint64_t gs;
 };
-#define HEADERS_STRUCT struct user_regs_struct_64
+
+union user_regs_t {
+    struct user_regs_32 regs32;
+    struct user_regs_64 regs64;
+};
 #endif /* defined(__i386__) || defined(__x86_64__) */
 
 #if defined(__arm__) || defined(__aarch64__)
-#ifndef ARM_pc
-#ifdef __ANDROID__ /* Building with NDK headers */
-#define ARM_pc uregs[15]
-#else /* Building with glibc headers */
-#define ARM_pc 15
-#endif
-#endif /* ARM_pc */
-#ifndef ARM_cpsr
-#ifdef __ANDROID__ /* Building with NDK headers */
-#define ARM_cpsr uregs[16]
-#else /* Building with glibc headers */
-#define ARM_cpsr 16
-#endif
-#endif /* ARM_cpsr */
-struct user_regs_struct_32 {
-    uint32_t uregs[18];
+struct user_regs_32 {
+    uint32_t r0;
+    uint32_t r1;
+    uint32_t r2;
+    uint32_t r3;
+    uint32_t r4;
+    uint32_t r5;
+    uint32_t r6;
+    uint32_t r7;
+    uint32_t r8;
+    uint32_t r9;
+    uint32_t r10;
+    uint32_t fp;
+    uint32_t ip;
+    uint32_t sp;
+    uint32_t lr;
+    uint32_t pc;
+    uint32_t cpsr;
+    uint32_t ORIG_r0;
 };
 
-struct user_regs_struct_64 {
+struct user_regs_64 {
     uint64_t regs[31];
     uint64_t sp;
     uint64_t pc;
     uint64_t pstate;
 };
-#define HEADERS_STRUCT struct user_regs_struct_64
+
+union user_regs_t {
+    struct user_regs_32 regs32;
+    struct user_regs_64 regs64;
+};
 #endif /* defined(__arm__) || defined(__aarch64__) */
 
 #if defined(__powerpc64__) || defined(__powerpc__)
-#define HEADERS_STRUCT struct pt_regs
-struct user_regs_struct_32 {
+struct user_regs_32 {
     uint32_t gpr[32];
     uint32_t nip;
     uint32_t msr;
@@ -181,7 +193,8 @@ struct user_regs_struct_32 {
     uint32_t zero2;
     uint32_t zero3;
 };
-struct user_regs_struct_64 {
+
+struct user_regs_64 {
     uint64_t gpr[32];
     uint64_t nip;
     uint64_t msr;
@@ -204,10 +217,16 @@ struct user_regs_struct_64 {
     uint64_t zero2;
     uint64_t zero3;
 };
+
+union user_regs_t {
+    struct user_regs_32 regs32;
+    struct user_regs_64 regs64;
+};
+
 #endif /* defined(__powerpc64__) || defined(__powerpc__) */
 
 #if defined(__mips__) || defined(__mips64__)
-struct user_regs_struct {
+struct user_regs_64 {
     uint64_t regs[32];
 
     uint64_t lo;
@@ -217,19 +236,118 @@ struct user_regs_struct {
     uint64_t cp0_status;
     uint64_t cp0_cause;
 };
-#define HEADERS_STRUCT struct user_regs_struct
+
+/*
+ * Despite what mips linux kernel headers/source code says, mips32 uses the same register laytout as
+ * mips64 only with GETREGS. With GETREGSET(NT_PRSTATUS) the structure size is 180 bytes
+ * (ELF_NGREG=45 * sizeof(uint32_t)=4 = 180). It also uses 24-byte padding for some,
+ * not-entierely-clear, reasons. The structure itself is not defined in the kernel, but only through
+ * how mips_dump_regs32() saves those registers via via MIPS32_EF_* defines.
+ */
+struct user_regs_32 {
+    uint32_t pad0[6];
+
+    uint32_t regs[32];
+
+    uint32_t lo;
+    uint32_t hi;
+    uint32_t cp0_epc;
+    uint32_t cp0_badvaddr;
+    uint32_t cp0_status;
+    uint32_t cp0_cause;
+    uint32_t unused0;
+};
+
+union user_regs_t {
+    struct user_regs_32 regs32;
+    struct user_regs_64 regs64;
+};
 #endif /* defined(__mips__) || defined(__mips64__) */
 
-#if defined(__ANDROID__)
-/*
- * Some Android ABIs don't implement PTRACE_GETREGS (e.g. aarch64)
- */
-#if defined(PTRACE_GETREGS)
-#define PTRACE_GETREGS_AVAILABLE 1
-#else
-#define PTRACE_GETREGS_AVAILABLE 0
-#endif /* defined(PTRACE_GETREGS) */
-#endif /* defined(__ANDROID__) */
+#if defined(__riscv)
+struct user_regs_64 {
+    uint64_t epc;
+    uint64_t ra;
+    uint64_t sp;
+    uint64_t gp;
+    uint64_t tp;
+    uint64_t t0;
+    uint64_t t1;
+    uint64_t t2;
+    uint64_t s0;
+    uint64_t s1;
+    uint64_t a0;
+    uint64_t a1;
+    uint64_t a2;
+    uint64_t a3;
+    uint64_t a4;
+    uint64_t a5;
+    uint64_t a6;
+    uint64_t a7;
+    uint64_t s2;
+    uint64_t s3;
+    uint64_t s4;
+    uint64_t s5;
+    uint64_t s6;
+    uint64_t s7;
+    uint64_t s8;
+    uint64_t s9;
+    uint64_t s10;
+    uint64_t s11;
+    uint64_t t3;
+    uint64_t t4;
+    uint64_t t5;
+    uint64_t t6;
+    uint64_t status;
+    uint64_t badaddr;
+    uint64_t cause;
+    uint64_t orig_a0;
+};
+
+struct user_regs_32 {
+    uint32_t epc;
+    uint32_t ra;
+    uint32_t sp;
+    uint32_t gp;
+    uint32_t tp;
+    uint32_t t0;
+    uint32_t t1;
+    uint32_t t2;
+    uint32_t s0;
+    uint32_t s1;
+    uint32_t a0;
+    uint32_t a1;
+    uint32_t a2;
+    uint32_t a3;
+    uint32_t a4;
+    uint32_t a5;
+    uint32_t a6;
+    uint32_t a7;
+    uint32_t s2;
+    uint32_t s3;
+    uint32_t s4;
+    uint32_t s5;
+    uint32_t s6;
+    uint32_t s7;
+    uint32_t s8;
+    uint32_t s9;
+    uint32_t s10;
+    uint32_t s11;
+    uint32_t t3;
+    uint32_t t4;
+    uint32_t t5;
+    uint32_t t6;
+    uint32_t status;
+    uint32_t badaddr;
+    uint32_t cause;
+    uint32_t orig_a0;
+};
+
+union user_regs_t {
+    struct user_regs_32 regs32;
+    struct user_regs_64 regs64;
+};
+#endif /* defined(__riscv) */
 
 #if defined(__clang__)
 _Pragma("clang diagnostic push");
@@ -293,7 +411,7 @@ static size_t arch_getProcMem(pid_t pid, uint8_t* buf, size_t len, uint64_t pc)
     if (process_vm_readv(pid, &local_iov, 1, &remote_iov, 1, 0) == (ssize_t)len) {
         return len;
     }
-    // Debug if failed since it shouldn't happen very often
+    /* Debug if failed since it shouldn't happen very often */
     PLOG_D("process_vm_readv() failed");
 
     /*
@@ -319,18 +437,7 @@ static size_t arch_getProcMem(pid_t pid, uint8_t* buf, size_t len, uint64_t pc)
 }
 
 static size_t arch_getPC(pid_t pid, uint64_t* pc, uint64_t* status_reg HF_ATTR_UNUSED) {
-/*
- * Some old ARM android kernels are failing with PTRACE_GETREGS to extract
- * the correct register values if struct size is bigger than expected. As such the
- * 32/64-bit multiplexing trick is not working for them in case PTRACE_GETREGSET
- * fails or is not implemented. To cover such cases we explicitly define
- * the struct size to 32bit version for arm CPU.
- */
-#if defined(__arm__)
-    struct user_regs_struct_32 regs;
-#else
-    HEADERS_STRUCT regs;
-#endif
+    union user_regs_t  regs;
     const struct iovec pt_iov = {
         .iov_base = &regs,
         .iov_len  = sizeof(regs),
@@ -338,87 +445,54 @@ static size_t arch_getPC(pid_t pid, uint64_t* pc, uint64_t* status_reg HF_ATTR_U
 
     if (ptrace(PTRACE_GETREGSET, pid, NT_PRSTATUS, &pt_iov) == -1L) {
         PLOG_D("ptrace(PTRACE_GETREGSET) failed");
-
-// If PTRACE_GETREGSET fails, try PTRACE_GETREGS if available
-#if PTRACE_GETREGS_AVAILABLE
-        if (ptrace(PTRACE_GETREGS, pid, 0, &regs)) {
-            PLOG_D("ptrace(PTRACE_GETREGS) failed");
-            LOG_W("ptrace PTRACE_GETREGSET & PTRACE_GETREGS failed to extract target registers");
-            return 0;
-        }
-#else
         return 0;
-#endif
     }
+
 #if defined(__i386__) || defined(__x86_64__)
-    /*
-     * 32-bit
-     */
-    if (pt_iov.iov_len == sizeof(struct user_regs_struct_32)) {
-        struct user_regs_struct_32* r32 = (struct user_regs_struct_32*)&regs;
-        *pc                             = r32->eip;
-        *status_reg                     = r32->eflags;
+    /* 32-bit */
+    if (pt_iov.iov_len == sizeof(struct user_regs_32)) {
+        *pc         = regs.regs32.eip;
+        *status_reg = regs.regs32.eflags;
+        return pt_iov.iov_len;
+    }
+    /* 64-bit */
+    if (pt_iov.iov_len == sizeof(struct user_regs_64)) {
+        *pc         = regs.regs64.ip;
+        *status_reg = regs.regs64.flags;
         return pt_iov.iov_len;
     }
 
-    /*
-     * 64-bit
-     */
-    if (pt_iov.iov_len == sizeof(struct user_regs_struct_64)) {
-        struct user_regs_struct_64* r64 = (struct user_regs_struct_64*)&regs;
-        *pc                             = r64->ip;
-        *status_reg                     = r64->flags;
-        return pt_iov.iov_len;
-    }
     LOG_W("Unknown registers structure size: '%zd'", pt_iov.iov_len);
     return 0;
 #endif /* defined(__i386__) || defined(__x86_64__) */
 
 #if defined(__arm__) || defined(__aarch64__)
-    /*
-     * 32-bit
-     */
-    if (pt_iov.iov_len == sizeof(struct user_regs_struct_32)) {
-        struct user_regs_struct_32* r32 = (struct user_regs_struct_32*)&regs;
-#ifdef __ANDROID__
-        *pc         = r32->ARM_pc;
-        *status_reg = r32->ARM_cpsr;
-#else
-        *pc         = r32->uregs[ARM_pc];
-        *status_reg = r32->uregs[ARM_cpsr];
-#endif
+    /* 32-bit */
+    if (pt_iov.iov_len == sizeof(struct user_regs_32)) {
+        *pc         = regs.regs32.pc;
+        *status_reg = regs.regs32.cpsr;
+        return pt_iov.iov_len;
+    }
+    /* 64-bit */
+    if (pt_iov.iov_len == sizeof(struct user_regs_64)) {
+        *pc         = regs.regs64.pc;
+        *status_reg = regs.regs64.pstate;
         return pt_iov.iov_len;
     }
 
-    /*
-     * 64-bit
-     */
-    if (pt_iov.iov_len == sizeof(struct user_regs_struct_64)) {
-        struct user_regs_struct_64* r64 = (struct user_regs_struct_64*)&regs;
-        *pc                             = r64->pc;
-        *status_reg                     = r64->pstate;
-        return pt_iov.iov_len;
-    }
     LOG_W("Unknown registers structure size: '%zd'", pt_iov.iov_len);
     return 0;
 #endif /* defined(__arm__) || defined(__aarch64__) */
 
 #if defined(__powerpc64__) || defined(__powerpc__)
-    /*
-     * 32-bit
-     */
-    if (pt_iov.iov_len == sizeof(struct user_regs_struct_32)) {
-        struct user_regs_struct_32* r32 = (struct user_regs_struct_32*)&regs;
-        *pc                             = r32->nip;
+    /* 32-bit */
+    if (pt_iov.iov_len == sizeof(struct user_regs_32)) {
+        *pc = regs.regs32.nip;
         return pt_iov.iov_len;
     }
-
-    /*
-     * 64-bit
-     */
-    if (pt_iov.iov_len == sizeof(struct user_regs_struct_64)) {
-        struct user_regs_struct_64* r64 = (struct user_regs_struct_64*)&regs;
-        *pc                             = r64->nip;
+    /* 64-bit */
+    if (pt_iov.iov_len == sizeof(struct user_regs_64)) {
+        *pc = regs.regs64.nip;
         return pt_iov.iov_len;
     }
 
@@ -427,10 +501,33 @@ static size_t arch_getPC(pid_t pid, uint64_t* pc, uint64_t* status_reg HF_ATTR_U
 #endif /* defined(__powerpc64__) || defined(__powerpc__) */
 
 #if defined(__mips__) || defined(__mips64__)
-    *pc = regs.cp0_epc;
-    return pt_iov.iov_len;
+    if (pt_iov.iov_len == sizeof(struct user_regs_64)) {
+        *pc = regs.regs64.cp0_epc;
+        return pt_iov.iov_len;
+    }
+    if (pt_iov.iov_len == sizeof(struct user_regs_32)) {
+        *pc = regs.regs32.cp0_epc;
+        return pt_iov.iov_len;
+    }
+
+    LOG_W("Unknown registers structure size: '%zd'", pt_iov.iov_len);
+    return 0;
 #endif /* defined(__mips__) || defined(__mips64__) */
 
+#if defined(__riscv)
+    if (pt_iov.iov_len == sizeof(struct user_regs_64)) {
+        *pc = regs.regs64.epc;
+        return pt_iov.iov_len;
+    }
+    if (pt_iov.iov_len == sizeof(struct user_regs_32)) {
+        *pc = regs.regs32.epc;
+        return pt_iov.iov_len;
+    }
+
+    LOG_W("Unknown registers structure size: '%zd'", pt_iov.iov_len);
+    return 0;
+#endif /* defined(__riscv) */
+
     LOG_D("Unknown/unsupported CPU architecture");
     return 0;
 }
@@ -458,7 +555,7 @@ static void arch_getInstrStr(pid_t pid, uint64_t pc, uint64_t status_reg HF_ATTR
     cs_arch arch;
     cs_mode mode;
 #if defined(__arm__) || defined(__aarch64__)
-    arch = (pcRegSz == sizeof(struct user_regs_struct_64)) ? CS_ARCH_ARM64 : CS_ARCH_ARM;
+    arch = (pcRegSz == sizeof(struct user_regs_64)) ? CS_ARCH_ARM64 : CS_ARCH_ARM;
     if (arch == CS_ARCH_ARM) {
         mode = (status_reg & 0x20) ? CS_MODE_THUMB : CS_MODE_ARM;
     } else {
@@ -466,7 +563,7 @@ static void arch_getInstrStr(pid_t pid, uint64_t pc, uint64_t status_reg HF_ATTR
     }
 #elif defined(__i386__) || defined(__x86_64__)
     arch = CS_ARCH_X86;
-    mode = (pcRegSz == sizeof(struct user_regs_struct_64)) ? CS_MODE_64 : CS_MODE_32;
+    mode = (pcRegSz == sizeof(struct user_regs_64)) ? CS_MODE_64 : CS_MODE_32;
 #else
     LOG_E("Unknown/Unsupported Android CPU architecture");
 #endif
